"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

var _possibleConstructorReturn2 = _interopRequireDefault(require("@babel/runtime/helpers/possibleConstructorReturn"));

var _assertThisInitialized2 = _interopRequireDefault(require("@babel/runtime/helpers/assertThisInitialized"));

var _inherits2 = _interopRequireDefault(require("@babel/runtime/helpers/inherits"));

var _getPrototypeOf2 = _interopRequireDefault(require("@babel/runtime/helpers/getPrototypeOf"));

var _get2 = _interopRequireDefault(require("@babel/runtime/helpers/get"));

var _react = _interopRequireDefault(require("react"));

var _uuid = require("uuid");

var _findIndex = _interopRequireDefault(require("lodash/findIndex"));

var _instructureValidations = require("instructure-validations");

var _uiElements = require("@instructure/ui-elements");

var _InteractionType2 = _interopRequireDefault(require("../InteractionType"));

var _interaction_slugs = require("../../interaction_slugs");

var _validationHelpers = require("../../util/validationHelpers");

var _formatMessage = _interopRequireDefault(require("@instructure/quiz-i18n/lib/format-message"));

// ================
// ================
//     HELPERS
// ================
// ================
function newId() {
  return (0, _uuid.v4)();
} // ================
// ================
//   VALIDATIONS
// ================
// ================


function scoringDatumValidator(val) {
  var editDistanceRules = [];

  if (val.scoringAlgorithm === 'TextCloseEnough') {
    editDistanceRules = [(0, _instructureValidations.rule)('numeric', {
      message: (0, _formatMessage.default)('Levenshtein Distance must be a number'),
      minMessage: (0, _formatMessage.default)('Levenshtein Distance must be greater than zero'),
      maxMessage: (0, _formatMessage.default)('Levenshtein Distance must be less than the length of the blank text'),
      requireInteger: true,
      integerMessage: (0, _formatMessage.default)('Levenshtein Distance must be an integer'),
      min: 1,
      max: val.scoringData.blankText.length
    })];
  }

  var textInChoicesRules = val.scoringAlgorithm === 'TextInChoices' // each is for multiple specified correct answers
  ? [(0, _instructureValidations.each)((0, _instructureValidations.rule)('presence', {
    message: (0, _validationHelpers.presenceMessage)((0, _formatMessage.default)('Correct value'))
  })), (0, _instructureValidations.onSelf)((0, _instructureValidations.rule)('listSize', {
    minMessage: (0, _formatMessage.default)('You must have an answer'),
    min: 1,
    allowNullOrUndefined: true
  }))] : [(0, _instructureValidations.onSelf)((0, _instructureValidations.rule)('presence', {
    message: (0, _validationHelpers.presenceMessage)((0, _formatMessage.default)('Correct value'))
  }))];
  return {
    scoringData: {
      editDistance: editDistanceRules,
      value: textInChoicesRules
    }
  };
}

var eachBlankValidator = {
  choices: [(0, _instructureValidations.onSelf)((0, _instructureValidations.rule)('listSize', {
    minMessage: (0, _formatMessage.default)('You must have more than one choice'),
    min: 2,
    allowNullOrUndefined: true
  })), (0, _instructureValidations.rule)('noDuplicates', {
    field: 'itemBody',
    message: (0, _validationHelpers.noDuplicatesMessage)((0, _formatMessage.default)('Choice'))
  }), (0, _instructureValidations.each)({
    itemBody: [(0, _instructureValidations.rule)('presence', {
      message: (0, _validationHelpers.presenceMessage)((0, _formatMessage.default)('Choice'))
    })]
  })]
};
var allBlanksValidator = (0, _instructureValidations.onSelf)((0, _instructureValidations.rule)('listSize', {
  minMessage: (0, _formatMessage.default)('You must have a blank'),
  min: 1
})); // ================
// ================
//      RECORD
// ================
// ================

var FillBlankInteractionType =
/*#__PURE__*/
function (_InteractionType) {
  (0, _inherits2.default)(FillBlankInteractionType, _InteractionType);

  function FillBlankInteractionType(obj) {
    var _this;

    (0, _classCallCheck2.default)(this, FillBlankInteractionType);
    _this = (0, _possibleConstructorReturn2.default)(this, (0, _getPrototypeOf2.default)(FillBlankInteractionType).call(this));
    _this.slug = _interaction_slugs.FILL_BLANK_SLUG;
    _this.hideResultStem = true;
    _this.translatedName = (0, _formatMessage.default)('Fill in the Blank');

    _this.getDefaultScoringData = function () {
      return {
        value: []
      };
    };

    _this.getDefaultInteractionData = function () {
      return {
        stemItems: [{
          id: newId(),
          type: 'text',
          value: '',
          position: 1
        }],
        blanks: []
      };
    };

    _this.getRenderedResponse = function (responseValue) {
      var interactionData = arguments.length > 1 && arguments[1] !== void 0 ? arguments[1] : {
        stemItems: [],
        blanks: []
      };
      return _react.default.createElement(_uiElements.Text, {
        color: "primary"
      }, responseValue.map(function (value) {
        var blankPosition = (0, _findIndex.default)(interactionData.blanks, function (blank) {
          return blank.id === value.id;
        });
        var blank = blankPosition >= 0 && interactionData.blanks[blankPosition];
        var valueText = '';

        if (blank && blank.answerType === 'openEntry') {
          valueText = value.value;
        } else if (blank) {
          var blankChoice = blank.choices.find(function (choice) {
            return choice.id === value.value;
          });
          valueText = blankChoice ? blankChoice.itemBody : '';
        }

        return (0, _formatMessage.default)('Blank {blankPosition, number}: {valueText}', {
          blankPosition: blankPosition >= 0 ? blankPosition + 1 : '',
          valueText: valueText
        });
      }).join(', '));
    };

    (0, _get2.default)((0, _getPrototypeOf2.default)(FillBlankInteractionType.prototype), "initializeProps", (0, _assertThisInitialized2.default)(_this)).call((0, _assertThisInitialized2.default)(_this), obj);
    return _this;
  }

  (0, _createClass2.default)(FillBlankInteractionType, [{
    key: "getDefaultUserResponse",
    value: function getDefaultUserResponse() {
      return {
        value: []
      };
    }
  }, {
    key: "hasResponse",
    value: function hasResponse(resp, intData) {
      var numBlanks = intData.blanks.length;
      var numWithResponse = resp.filter(function (blank) {
        return blank.value;
      }).length;
      return numBlanks === numWithResponse;
    }
  }]);
  return FillBlankInteractionType;
}(_InteractionType2.default);

exports.default = FillBlankInteractionType;
FillBlankInteractionType.validations = {
  interactionData: {
    blanks: [allBlanksValidator, (0, _instructureValidations.each)(eachBlankValidator)]
  },
  scoringData: {
    value: [(0, _instructureValidations.each)(scoringDatumValidator)]
  }
};